@@ -3,5 +3,5 @@ library(Zelig)
 library(survey)
 library(dplyr)
 
-set.seed("123")
+set.seed(123)
 test_check("Zelig")
