@@ -2,6 +2,6 @@
 
 test_that('REQUIRE TEST tobitbayes Monte Carlo', {
     z <- ztobitbayes$new()
-    test <- z$mcunit(nsim=1000, minx=-1, plot = FALSE)
-    expect_true(test)
+    test.tobitbayes <- z$mcunit(nsim=2000, ci=0.99, minx=0, plot = FALSE)
+    expect_true(test.tobitbayes)
 })
\ No newline at end of file
