@@ -12,10 +12,10 @@ test_that('REQUIRE TEST weighting', {
 	w2 <- rep(c(1.8,0.2), 45)
 
 	z1.out <- zelig( y ~ x, cite=FALSE, model="ls", weights="w1", data=mydata)
-	expect_equivalent(length(z1.out$getcoef()[[1]]),2)
+	expect_equivalent(length(z1.out$get_coef()[[1]]),2)
 
 	z2.out <- zelig( y ~ x, cite=FALSE, model="ls", weights=w2, data=mydata)
-	expect_equivalent(length(z2.out$getcoef()[[1]]),2)
+	expect_equivalent(length(z2.out$get_coef()[[1]]),2)
 
 	z3.out <- zls$new()
 	expect_warning(z3.out$zelig( y ~ x, weights="noSuchName", data=mydata))
@@ -25,10 +25,10 @@ test_that('REQUIRE TEST weighting', {
 
 	continuous.weights <- rep(x=c(0.6, 1, 1.4), times=30)
 	z5.out <- zelig( z ~ x, model="logit", weights=continuous.weights, data=mydata)
-	expect_equivalent(length(z5.out$getcoef()[[1]]),2)
+	expect_equivalent(length(z5.out$get_coef()[[1]]),2)
 
 	integer.weights <- rep(x=c(0, 1, 2), times=30)
 	z6.out <- zelig( z ~ x, model="logit", weights=integer.weights, data=mydata)
-	expect_equivalent(length(z6.out$getcoef()[[1]]),2)
+	expect_equivalent(length(z6.out$get_coef()[[1]]),2)
 
 })
\ No newline at end of file
